package jp.sf.amateras.xmleditor.editors;

import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.Region;
import org.eclipse.jface.text.hyperlink.IHyperlink;
import org.eclipse.jface.text.hyperlink.IHyperlinkDetector;

public class ClassNameHyperlinkDetector implements IHyperlinkDetector {
	
	private IJavaProject project;
	
	public ClassNameHyperlinkDetector(IJavaProject project){
		this.project = project;
	}
	
	public IHyperlink[] detectHyperlinks(ITextViewer textViewer,
			IRegion region, boolean canShowMultipleHyperlinks) {
		
		String source = textViewer.getDocument().get();
		int offset = region.getOffset();
		
		// extracts the word at the caret position
		String word = String.valueOf(source.charAt(offset));
		int index = offset -1;
		while(index >= 0){
			char c = source.charAt(index);
			if(Character.isJavaIdentifierPart(c) || c=='.'){
				word = c + word;
				index--;
			} else {
				break;
			}
		}
		index = offset + 1;
		offset = offset - word.length() + 1;
		while(source.length()>index){
			char c = source.charAt(index);
			if(Character.isJavaIdentifierPart(c) || c=='.'){
				word = word + c;
				index++;
			} else {
				break;
			}
		}
		
		try {
			IType type = project.findType(word);
			if(type!=null && type.exists()){
				return new IHyperlink[]{
						new ClassNameHyperlink(type, new Region(offset, word.length()))
				};
			}
		} catch (JavaModelException e) {
		}
		
		return null;
	}

}
