package jp.sf.amateras.xmleditor.editors;

import java.util.ArrayList;
import java.util.List;

import jp.aonir.fuzzyxml.FuzzyXMLDocument;
import jp.aonir.fuzzyxml.FuzzyXMLElement;
import jp.aonir.fuzzyxml.FuzzyXMLNode;
import jp.aonir.fuzzyxml.FuzzyXMLParser;
import jp.sf.amateras.xmleditor.XMLEditorPlugin;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;

public class XMLOutlinePage extends ContentOutlinePage {
	
	private RootModel root = new RootModel();
	private XMLEditor editor = null;
	
	public XMLOutlinePage(XMLEditor editor){
		this.editor = editor;
	}
	
	public void createControl(Composite parent) {
		super.createControl(parent);
		
		TreeViewer viewer = getTreeViewer();
		viewer.setContentProvider(new XMLContentProvider());
		viewer.setLabelProvider(new XMLLabelProvider());
		
		viewer.addSelectionChangedListener(new ISelectionChangedListener(){
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection sel = (IStructuredSelection)event.getSelection();
				Object element = sel.getFirstElement();
				if(element instanceof FuzzyXMLNode){
					int offset = ((FuzzyXMLNode)element).getOffset();
					editor.selectAndReveal(offset, 0);
				}
			}
		});
		
		viewer.setInput(root);
		
		refresh();
	}
	
	public void refresh(){
		IDocumentProvider provider = editor.getDocumentProvider();
		IDocument document = provider.getDocument(editor.getEditorInput());
		String source = document.get();
		FuzzyXMLDocument doc = new FuzzyXMLParser().parse(source);
		//getTreeViewer().setInput(doc.getDocumentElement());
		root.documentElement = doc.getDocumentElement();
		getTreeViewer().refresh();
	}
	
	private class RootModel {
		private FuzzyXMLElement documentElement;
	}
	
	private class XMLContentProvider implements ITreeContentProvider {
		
		public Object[] getChildren(Object parentElement) {
			if(parentElement instanceof FuzzyXMLElement){
				FuzzyXMLNode[] nodes = ((FuzzyXMLElement)parentElement).getChildren();
				List children = new ArrayList();
				for(int i=0;i<nodes.length;i++){
					if(nodes[i] instanceof FuzzyXMLElement){
						children.add(nodes[i]);
					}
				}
				return children.toArray();
			}
			return new Object[0];
		}
		
		public Object getParent(Object element) {
			if(element instanceof FuzzyXMLElement){
				return ((FuzzyXMLElement)element).getParentNode();
			}
			return null;
		}
		
		public boolean hasChildren(Object element) {
			if(getChildren(element).length==0){
				return false;
			} else {
				return true;
			}
		}
		
		public Object[] getElements(Object inputElement) {
			return getChildren(root.documentElement);
		}
		
		public void dispose() {
		}
		
		public void inputChanged(Viewer viewer, Object oldInput,Object newInput) {
		}
	}
	
	private class XMLLabelProvider extends LabelProvider {
		
		private Image elementImage 
			= XMLEditorPlugin.getImageDescriptor("icons/element.gif").createImage();
		
		public Image getImage(Object element) {
			if(element instanceof FuzzyXMLElement){
				return elementImage;
			}
			return null;
		}
		
		public String getText(Object element) {
			if(element instanceof FuzzyXMLElement){
				return ((FuzzyXMLElement)element).getName();
			}
			return super.getText(element);
		}

		public void dispose() {
			elementImage.dispose();
			super.dispose();
		}
	}

}
